# Config file for the eckit package
# Defines the following variables:
#
#  ECKIT_INCLUDE_DIRS   - include directories
#  ECKIT_DEFINITIONS    - preprocessor definitions
#  ECKIT_LIBRARIES      - libraries to link against
#  ECKIT_FEATURES       - list of enabled features
#  ECKIT_VERSION        - version of the package
#  ECKIT_GIT_SHA1       - Git revision of the package
#  ECKIT_GIT_SHA1_SHORT - short Git revision of the package
#
# Also defines eckit third-party library dependencies:
#  ECKIT_TPLS             - package names of  third-party library dependencies
#  ECKIT_TPL_INCLUDE_DIRS - include directories
#  ECKIT_TPL_DEFINITIONS  - preprocessor definitions
#  ECKIT_TPL_LIBRARIES    - libraries to link against

### compute paths

get_filename_component(ECKIT_CMAKE_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

set( ECKIT_SELF_INCLUDE_DIRS "${ECKIT_CMAKE_DIR}/../../../include" )
set( ECKIT_SELF_DEFINITIONS  "_FILE_OFFSET_BITS=64" )
set( ECKIT_SELF_LIBRARIES    "eckit;eckit_geometry;eckit_linalg;eckit_maths;eckit_web;eckit_cmd" )

set( ECKIT_TPLS              "MPI;LEXYACC;Curses;Eigen3;LAPACK;OpenSSL;CURL" )
set( ECKIT_TPL_INCLUDE_DIRS  "/usr/include;/usr/include/eigen3;/usr/include/arm-linux-gnueabi" )
set( ECKIT_TPL_DEFINITIONS   "" )
set( ECKIT_TPL_LIBRARIES     "/usr/lib/arm-linux-gnueabi/openmpi/lib/libmpi_cxx.so;/usr/lib/arm-linux-gnueabi/openmpi/lib/libmpi.so;/usr/lib/arm-linux-gnueabi/libcurses.so;/usr/lib/arm-linux-gnueabi/libform.so;/usr/lib/arm-linux-gnueabi/liblapack.so;/usr/lib/arm-linux-gnueabi/libblas.so;/usr/lib/arm-linux-gnueabi/libssl.so;/usr/lib/arm-linux-gnueabi/libcrypto.so;/usr/lib/arm-linux-gnueabi/libcurl.so" )

set( ECKIT_VERSION           "0.24.0" )
set( ECKIT_GIT_SHA1          "" )
set( ECKIT_GIT_SHA1_SHORT    "" )

### export include paths as absolute paths

set( ECKIT_INCLUDE_DIRS "" )
foreach( path ${ECKIT_SELF_INCLUDE_DIRS} )
  get_filename_component( abspath ${path} ABSOLUTE )
  list( APPEND ECKIT_INCLUDE_DIRS ${abspath} )
endforeach()
list( APPEND ECKIT_INCLUDE_DIRS ${ECKIT_TPL_INCLUDE_DIRS} )

### export definitions

set( ECKIT_DEFINITIONS      ${ECKIT_SELF_DEFINITIONS} ${ECKIT_TPL_DEFINITIONS} )

### export list of all libraries

set( ECKIT_LIBRARIES        ${ECKIT_SELF_LIBRARIES}   ${ECKIT_TPL_LIBRARIES}   )

### export the features provided by the package

set( ECKIT_FEATURES    "TESTS;MPI;ECKIT_CMD;EIGEN;LAPACK;SSL;CURL;DOCS" )
foreach( _f ${ECKIT_FEATURES} )
  set( ECKIT_HAVE_${_f} 1 )
endforeach()

# Has this configuration been exported from a build tree?
set( ECKIT_IS_BUILD_DIR_EXPORT OFF )

if( EXISTS ${ECKIT_CMAKE_DIR}/eckit-import.cmake )
  set( ECKIT_IMPORT_FILE "${ECKIT_CMAKE_DIR}/eckit-import.cmake" )
  include( ${ECKIT_IMPORT_FILE} )
endif()

# here goes the imports of the TPL's

include( ${CMAKE_CURRENT_LIST_FILE}.tpls OPTIONAL )

# insert definitions for IMPORTED targets

if( NOT eckit_BINARY_DIR )

  if( ECKIT_IS_BUILD_DIR_EXPORT )
    include( "/build/odb-api/debian/build/odb_api_bundle-targets.cmake" OPTIONAL )
  else()
    include( "${ECKIT_CMAKE_DIR}/eckit-targets.cmake" OPTIONAL )
  endif()

endif()

# publish this file as imported

set( ECKIT_IMPORT_FILE ${CMAKE_CURRENT_LIST_FILE} )
mark_as_advanced( ECKIT_IMPORT_FILE )

# set eckit_BASE_DIR for final installations or build directories

if( NOT eckit )
  if( ECKIT_IS_BUILD_DIR_EXPORT )
    set( eckit_BASE_DIR /build/odb-api/debian/build )
  else()
    get_filename_component( abspath ${CMAKE_CURRENT_LIST_DIR}/../../.. ABSOLUTE )
    set( eckit_BASE_DIR ${abspath} )
  endif()
endif()
