# Config file for the metkit package
# Defines the following variables:
#
#  METKIT_INCLUDE_DIRS   - include directories
#  METKIT_DEFINITIONS    - preprocessor definitions
#  METKIT_LIBRARIES      - libraries to link against
#  METKIT_FEATURES       - list of enabled features
#  METKIT_VERSION        - version of the package
#  METKIT_GIT_SHA1       - Git revision of the package
#  METKIT_GIT_SHA1_SHORT - short Git revision of the package
#
# Also defines metkit third-party library dependencies:
#  METKIT_TPLS             - package names of  third-party library dependencies
#  METKIT_TPL_INCLUDE_DIRS - include directories
#  METKIT_TPL_DEFINITIONS  - preprocessor definitions
#  METKIT_TPL_LIBRARIES    - libraries to link against

### compute paths

get_filename_component(METKIT_CMAKE_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

set( METKIT_SELF_INCLUDE_DIRS "${METKIT_CMAKE_DIR}/../../../include" )
set( METKIT_SELF_DEFINITIONS  "_FILE_OFFSET_BITS=64" )
set( METKIT_SELF_LIBRARIES    "metkit" )

set( METKIT_TPLS              "eccodes;eckit" )
set( METKIT_TPL_INCLUDE_DIRS  "/usr/lib/include;/usr/include;/usr/lib/python3/dist-packages/numpy/core/include;/usr/include/openjpeg-2.1;/build/odb-api/eckit/src;/build/odb-api/debian/build/eckit/src;/usr/include/eigen3;/usr/include/arm-linux-gnueabi" )
set( METKIT_TPL_DEFINITIONS   "" )
set( METKIT_TPL_LIBRARIES     "eccodes;eccodes_f90;/usr/lib/arm-linux-gnueabi/libpng.so;/usr/lib/arm-linux-gnueabi/libz.so;/usr/lib/arm-linux-gnueabi/libaec.so;/usr/lib/arm-linux-gnueabi/libm.so;/usr/lib/arm-linux-gnueabi/libopenjp2.so;eckit;eckit_geometry;eckit_linalg;eckit_maths;eckit_web;eckit_cmd" )

set( METKIT_VERSION           "0.16.0" )
set( METKIT_GIT_SHA1          "" )
set( METKIT_GIT_SHA1_SHORT    "" )

### export include paths as absolute paths

set( METKIT_INCLUDE_DIRS "" )
foreach( path ${METKIT_SELF_INCLUDE_DIRS} )
  get_filename_component( abspath ${path} ABSOLUTE )
  list( APPEND METKIT_INCLUDE_DIRS ${abspath} )
endforeach()
list( APPEND METKIT_INCLUDE_DIRS ${METKIT_TPL_INCLUDE_DIRS} )

### export definitions

set( METKIT_DEFINITIONS      ${METKIT_SELF_DEFINITIONS} ${METKIT_TPL_DEFINITIONS} )

### export list of all libraries

set( METKIT_LIBRARIES        ${METKIT_SELF_LIBRARIES}   ${METKIT_TPL_LIBRARIES}   )

### export the features provided by the package

set( METKIT_FEATURES    "TESTS;MPI;ECKIT_CMD;EIGEN;LAPACK;SSL;CURL;DOCS;GRIB;NETCDF" )
foreach( _f ${METKIT_FEATURES} )
  set( METKIT_HAVE_${_f} 1 )
endforeach()

# Has this configuration been exported from a build tree?
set( METKIT_IS_BUILD_DIR_EXPORT OFF )

if( EXISTS ${METKIT_CMAKE_DIR}/metkit-import.cmake )
  set( METKIT_IMPORT_FILE "${METKIT_CMAKE_DIR}/metkit-import.cmake" )
  include( ${METKIT_IMPORT_FILE} )
endif()

# here goes the imports of the TPL's

include( ${CMAKE_CURRENT_LIST_FILE}.tpls OPTIONAL )

# insert definitions for IMPORTED targets

if( NOT metkit_BINARY_DIR )

  if( METKIT_IS_BUILD_DIR_EXPORT )
    include( "/build/odb-api/debian/build/odb_api_bundle-targets.cmake" OPTIONAL )
  else()
    include( "${METKIT_CMAKE_DIR}/metkit-targets.cmake" OPTIONAL )
  endif()

endif()

# publish this file as imported

set( METKIT_IMPORT_FILE ${CMAKE_CURRENT_LIST_FILE} )
mark_as_advanced( METKIT_IMPORT_FILE )

# set metkit_BASE_DIR for final installations or build directories

if( NOT metkit )
  if( METKIT_IS_BUILD_DIR_EXPORT )
    set( metkit_BASE_DIR /build/odb-api/debian/build )
  else()
    get_filename_component( abspath ${CMAKE_CURRENT_LIST_DIR}/../../.. ABSOLUTE )
    set( metkit_BASE_DIR ${abspath} )
  endif()
endif()
