# Config file for the odb_api package
# Defines the following variables:
#
#  ODB_API_INCLUDE_DIRS   - include directories
#  ODB_API_DEFINITIONS    - preprocessor definitions
#  ODB_API_LIBRARIES      - libraries to link against
#  ODB_API_FEATURES       - list of enabled features
#  ODB_API_VERSION        - version of the package
#  ODB_API_GIT_SHA1       - Git revision of the package
#  ODB_API_GIT_SHA1_SHORT - short Git revision of the package
#
# Also defines odb_api third-party library dependencies:
#  ODB_API_TPLS             - package names of  third-party library dependencies
#  ODB_API_TPL_INCLUDE_DIRS - include directories
#  ODB_API_TPL_DEFINITIONS  - preprocessor definitions
#  ODB_API_TPL_LIBRARIES    - libraries to link against

### compute paths

get_filename_component(ODB_API_CMAKE_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

set( ODB_API_SELF_INCLUDE_DIRS "${ODB_API_CMAKE_DIR}/../../../include" )
set( ODB_API_SELF_DEFINITIONS  "_FILE_OFFSET_BITS=64" )
set( ODB_API_SELF_LIBRARIES    "Odb_fortran;Odb" )

set( ODB_API_TPLS              "eckit;metkit;OMP;Python;SWIG" )
set( ODB_API_TPL_INCLUDE_DIRS  "/build/odb-api/eckit/src;/build/odb-api/debian/build/eckit/src;/usr/include;/usr/include/eigen3;/usr/include/arm-linux-gnueabi;/build/odb-api/metkit/src;/build/odb-api/debian/build/metkit/src;/usr/lib/include;/usr/lib/python3/dist-packages/numpy/core/include;/usr/include/openjpeg-2.1;/usr/include/python3.7m" )
set( ODB_API_TPL_DEFINITIONS   "" )
set( ODB_API_TPL_LIBRARIES     "eckit;eckit_geometry;eckit_linalg;eckit_maths;eckit_web;eckit_cmd;metkit;-L/usr/lib/python3.7/config-3.7m-arm-linux-gnueabi -L/usr/lib -lpython3.7m -lpthread -ldl  -lutil -lm  -Xlinker -export-dynamic -Wl,-O1 -Wl,-Bsymbolic-functions" )

set( ODB_API_VERSION           "0.18.1" )
set( ODB_API_GIT_SHA1          "" )
set( ODB_API_GIT_SHA1_SHORT    "" )

### export include paths as absolute paths

set( ODB_API_INCLUDE_DIRS "" )
foreach( path ${ODB_API_SELF_INCLUDE_DIRS} )
  get_filename_component( abspath ${path} ABSOLUTE )
  list( APPEND ODB_API_INCLUDE_DIRS ${abspath} )
endforeach()
list( APPEND ODB_API_INCLUDE_DIRS ${ODB_API_TPL_INCLUDE_DIRS} )

### export definitions

set( ODB_API_DEFINITIONS      ${ODB_API_SELF_DEFINITIONS} ${ODB_API_TPL_DEFINITIONS} )

### export list of all libraries

set( ODB_API_LIBRARIES        ${ODB_API_SELF_LIBRARIES}   ${ODB_API_TPL_LIBRARIES}   )

### export the features provided by the package

set( ODB_API_FEATURES    "TESTS;MPI;ECKIT_CMD;EIGEN;LAPACK;SSL;CURL;DOCS;GRIB;NETCDF;FORTRAN;ODB_API_SERVER_SIDE;OMP;PYTHON" )
foreach( _f ${ODB_API_FEATURES} )
  set( ODB_API_HAVE_${_f} 1 )
endforeach()

# Has this configuration been exported from a build tree?
set( ODB_API_IS_BUILD_DIR_EXPORT OFF )

if( EXISTS ${ODB_API_CMAKE_DIR}/odb_api-import.cmake )
  set( ODB_API_IMPORT_FILE "${ODB_API_CMAKE_DIR}/odb_api-import.cmake" )
  include( ${ODB_API_IMPORT_FILE} )
endif()

# here goes the imports of the TPL's

include( ${CMAKE_CURRENT_LIST_FILE}.tpls OPTIONAL )

# insert definitions for IMPORTED targets

if( NOT odb_api_BINARY_DIR )

  if( ODB_API_IS_BUILD_DIR_EXPORT )
    include( "/build/odb-api/debian/build/odb_api_bundle-targets.cmake" OPTIONAL )
  else()
    include( "${ODB_API_CMAKE_DIR}/odb_api-targets.cmake" OPTIONAL )
  endif()

endif()

# publish this file as imported

set( ODB_API_IMPORT_FILE ${CMAKE_CURRENT_LIST_FILE} )
mark_as_advanced( ODB_API_IMPORT_FILE )

# set odb_api_BASE_DIR for final installations or build directories

if( NOT odb_api )
  if( ODB_API_IS_BUILD_DIR_EXPORT )
    set( odb_api_BASE_DIR /build/odb-api/debian/build )
  else()
    get_filename_component( abspath ${CMAKE_CURRENT_LIST_DIR}/../../.. ABSOLUTE )
    set( odb_api_BASE_DIR ${abspath} )
  endif()
endif()
